#ifndef _CONFIG_PARAM_H
#define _CONFIG_PARAM_H

#include <stdlib.h>

/**
 * File created on 2003-05-24 by Jeko.
 * (c)2003, JC Hoelt for iOS-software.
 *
 * LGPL Licence.
 * If you use this file on a visual program,
 *    please make my name being visible on it.
 */

#define PARAM_INTVAL 'i'
#define PARAM_FLOATVAL 'f'
#define PARAM_BOOLVAL 'b'
#define PARAM_STRVAL 's'
#define PARAM_LISTVAL 'l'

struct IntVal {
  int value;
  int min;
  int max;
  int step;
};
struct FloatVal {
  float value;
  float min;
  float max;
  float step;
};
struct StrVal {
  char *value;
};
struct ListVal {
  char *value;
  int nbChoices;
  char **choices;
};
struct BoolVal {
  int value;
};


typedef struct _PARAM {
  char *name;
  char *desc;
  char type;
  union {
    struct IntVal ival;
    struct FloatVal fval;
    struct StrVal sval;
    struct ListVal slist;
    struct BoolVal bval;
  } param;
  
  /* used by the core to inform the GUI of a change */
  void (*change_listener)(struct _PARAM *_this);

  /* used by the GUI to inform the core of a change */
  void (*changed)(struct _PARAM *_this);
  
  void *user_data; /* can be used by the GUI */
} PluginParam;

#define IVAL(p) ((p).param.ival.value)
#define SVAL(p) ((p).param.sval.value)
#define FVAL(p) ((p).param.fval.value)
#define BVAL(p) ((p).param.bval.value)
#define LVAL(p) ((p).param.slist.value)

#define FMIN(p) ((p).param.fval.min)
#define FMAX(p) ((p).param.fval.max)
#define FSTEP(p) ((p).param.fval.step)

#define IMIN(p) ((p).param.ival.min)
#define IMAX(p) ((p).param.ival.max)
#define ISTEP(p) ((p).param.ival.step)

static void empty_fct(){}

static PluginParam secure_param() {
  PluginParam p;
  p.changed = empty_fct;
  p.change_listener = empty_fct;
  p.user_data = 0;
  p.name = p.desc = 0;
  return p;
}

static PluginParam secure_f_param(char *name) {
  PluginParam p = secure_param();
  p.name = name;
  p.type = PARAM_FLOATVAL;
  FMIN(p) = 0;
  FMAX(p) = 1;
  FSTEP(p) = 0.01;
  return p;
}

static PluginParam secure_i_param(char *name) {
  PluginParam p = secure_param();
  p.name = name;
  p.type = PARAM_INTVAL;
  return p;
}

typedef struct _PARAMETERS {
  char *name;
  char *desc;
  int nbParams;
  PluginParam **params;
} PluginParameters;

static PluginParameters plugin_parameters(const char *name, int nb) {
  PluginParameters p;
  p.name = (char *)name;
  p.desc = "";
  p.nbParams = nb;
  p.params = (PluginParam**)malloc(nb*sizeof(PluginParam*));
  return p;
}

#endif
