#include "convolve_fx.h"

#define MAX 2.0f

typedef struct _CONV_DATA{
  float factor;
  int lock;
  int started;
  float curPower;
  PluginParam light;
  PluginParam factor_adj_p;
  PluginParam factor_p;
  PluginParameters params;
} ConvData;

static void convolve_init(VisualFX *_this) {
  ConvData *data;
  data = (ConvData*)malloc(sizeof(ConvData));
  data->factor = 0.0f;
  data->lock = 0;
  data->curPower = 0;
  data->started = 0;

  data->light = secure_param();
  data->light.name = "Screen Brightness";
  data->light.type = PARAM_INTVAL;
  data->light.param.ival.min = 0;
  data->light.param.ival.max = 300;
  data->light.param.ival.step = 1;
  data->light.param.ival.value = 100;

  data->factor_adj_p = secure_i_param("Flash Intensity");
  data->factor_adj_p.param.ival.min = 0;
  data->factor_adj_p.param.ival.max = 200;
  data->factor_adj_p.param.ival.step = 1;
  data->factor_adj_p.param.ival.value = 100;

  data->factor_p = secure_f_param("Factor");
  FVAL(data->factor_p) = data->factor / MAX;

  data->params = plugin_parameters ("Bright Flash", 4);
  data->params.params[0] = &data->light;
  data->params.params[1] = &data->factor_adj_p;
  data->params.params[2] = 0;
  data->params.params[3] = &data->factor_p;

  _this->params = &data->params;
  _this->fx_data = (void*)data;
}

static void convolve_free(VisualFX *_this) {
  free (_this->fx_data);
}

static void convolve_apply(VisualFX *_this, Pixel *src, Pixel *dest, PluginInfo *info) {

  ConvData *data = (ConvData*)_this->fx_data;
  int i;
  float ff;
  int iff;

  if ((data->lock) && (data->curPower > info->sound.goomPower))
    data->lock--;
  else {
    if ((info->sound.timeSinceLastGoom < 10)
	&& (data->started || ((info->sound.goomPower > 0.1) && (info->sound.speedvar > 0.14)))) {
      if ((!data->started) || (data->curPower < info->sound.goomPower))
	data->curPower = info->sound.goomPower;
      data->started = 1;
      data->lock = 0;

      data->factor += data->curPower * 2 * (info->sound.speedvar / 4 + 0.95);
      if (data->factor > MAX)
	data->factor = MAX;
    }
    else if (info->sound.timeSinceLastGoom == 10) {
      data->lock = 200;
      data->curPower = info->sound.goomPower;
      data->started = 0;
    }
  }
  FVAL(data->factor_p) = data->factor / MAX;
  data->factor_p.change_listener(&data->factor_p);

  data->factor *= 0.96f;

  ff = data->factor * (float)IVAL(data->factor_adj_p) / 100.0f + (float)IVAL(data->light) / 100.0f;

  iff = (int)(ff * 256);

  for (i=info->screen.size-1;i--;) {
    int c;
    for (c=4;c--;) {
      int f = (src[i].cop[c] * iff) >> 8;
      if (f & 0xffffff00) f=0xff;
      dest[i].cop[c] = (unsigned char)f;
    }
  }
}

VisualFX convolve_create() {
  VisualFX vfx = {
    init: convolve_init,
    free: convolve_free,
    apply: convolve_apply,
    fx_data: 0
  };
  return vfx;
}
