#include "convolve_fx.h"
#include "goom_tools.h"

#include "mathtools.h"

#undef HAVE_MMX
#include "drawmethods.h"

#define MODE 2

/* TODO : FAIRE PROPREMENT */
#define NCOL 15
static int colval[] = {
  0x1416181a,
  0x1419181a,
  0x141f181a,
  0x1426181a,
  0x142a181a,
  0x142f181a,
  0x1436181a,
  0x142f1819,
  0x14261615,
  0x13201411,
  0x111a100a,
  0x0c180508,
  0x08100304,
  0x00050101,
  0x0
};

typedef struct _FS_STAR {
  float x,y;
  float vx,vy;
  float ax,ay;
  float age,vage;
} Star;

typedef struct _FS_DATA{

  int nbStars;

  int maxStars;
  Star *stars;

  float min_age;
  float max_age;

  PluginParam density_p;
  PluginParam min_age_p;
  PluginParam max_age_p;
  PluginParam nbStars_p;
  PluginParam nbStars_limit_p;

  PluginParameters params;
} FSData;

static void fs_init(VisualFX *_this) {
  FSData *data;
  data = (FSData*)malloc(sizeof(FSData));

  data->maxStars = 4096;
  data->stars = (Star*)malloc(data->maxStars * sizeof(Star));
  data->nbStars = 0;

  data->params = plugin_parameters ("Fireworks/Stars", 7);

  data->density_p = secure_i_param ("Stars Density");
  IVAL(data->density_p) = 10;
  IMIN(data->density_p) = 0;
  IMAX(data->density_p) = 100;
  ISTEP(data->density_p) = 1;

  data->max_age_p = secure_i_param ("Smallest Bombs");
  IVAL(data->max_age_p) = 75;
  IMIN(data->max_age_p) = 0;
  IMAX(data->max_age_p) = 100;
  ISTEP(data->max_age_p) = 1;

  data->min_age_p = secure_i_param ("Largest Bombs");
  IVAL(data->min_age_p) = 98;
  IMIN(data->min_age_p) = 0;
  IMAX(data->min_age_p) = 100;
  ISTEP(data->min_age_p) = 1;

  data->nbStars_limit_p = secure_i_param ("Max Number of Particules");
  IVAL(data->nbStars_limit_p) = data->maxStars * 2 / 3;
  IMIN(data->nbStars_limit_p) = 0;
  IMAX(data->nbStars_limit_p) = data->maxStars;
  ISTEP(data->nbStars_limit_p) = 64;

  data->nbStars_p = secure_f_param ("Number of Particules (% of Max)");

  data->params.params[0] = &data->density_p;
  data->params.params[1] = &data->nbStars_limit_p;
  data->params.params[2] = 0;
  data->params.params[3] = &data->min_age_p;
  data->params.params[4] = &data->max_age_p;
  data->params.params[5] = 0;
  data->params.params[6] = &data->nbStars_p;

  _this->params = &data->params;
  _this->fx_data = (void*)data;
}

static void fs_free(VisualFX *_this) {
  free (_this->fx_data);
}

static void addAStar (FSData *fs, PluginInfo *info) {

  int i = fs->nbStars;
  if (fs->nbStars >= fs->maxStars)
    return;
  fs->nbStars++;

  if (MODE==1) {
    fs->stars[i].x = 0;
    fs->stars[i].y = iRAND(info->screen.height);
    fs->stars[i].vx = 0.3f+(float)iRAND(40)/20.0f;
    fs->stars[i].vy = 0;
    fs->stars[i].ax = 0;
    fs->stars[i].ay = 0;
    fs->stars[i].age = 0;
    fs->stars[i].vage = 0;
  }
  else if (MODE==2) {
    fs->stars[i].x = info->screen.width/2;
    fs->stars[i].y = info->screen.height/2;
    fs->stars[i].vx = (float)iRAND(40)/20.0f - (float)iRAND(40)/20.0f;
    fs->stars[i].vy = (float)iRAND(40)/20.0f - (float)iRAND(40)/20.0f;
    fs->stars[i].ax = 0;
    fs->stars[i].ay = 0.01f;
    fs->stars[i].age = 0;
    fs->stars[i].vage = 0.1;
  }
}

static void addABomb (FSData *fs, int mx, int my, float radius, float vage) {
  int i = fs->nbStars;
  float r1,r2;
  float rnd;

  if (fs->nbStars >= fs->maxStars)
    return;
  fs->nbStars++;

  fs->stars[i].x = mx;
  fs->stars[i].y = my;

  r1 = radius * (float)iRAND(100)/100.0f;
  if (iRAND(2)==1)
    r1=-r1;

  r2 = radius * (float)iRAND(100)/100.0f;
  if (iRAND(2)==1)
    r2=-r2;
  
  rnd = (float)iRAND(100)/100.0f + 1.0f;
  r1*=rnd;
  r2*=rnd;

  fs->stars[i].vx = r1;
  fs->stars[i].vy = -0.1f + r2;

  fs->stars[i].ax = 0;
  fs->stars[i].ay = 0.01f;
  
  fs->stars[i].age = 0;
  if (vage<fs->min_age) vage=fs->min_age;
  fs->stars[i].vage = vage;
}

static void updateStar (Star *s) {
  s->x+=s->vx;
  s->y+=s->vy;
  s->vx+=s->ax;
  s->vy+=s->ay;
  s->age+=s->vage;
}

static void fs_apply(VisualFX *_this, Pixel *src, Pixel *dest, PluginInfo *info) {

  int i;
  int col;
  FSData *data = (FSData*)_this->fx_data;

  data->min_age = 1.0f - (float)IVAL(data->min_age_p)/100.0f;
  data->max_age = 1.0f - (float)IVAL(data->max_age_p)/100.0f;
  FVAL(data->nbStars_p) = (float)data->nbStars / (float)data->maxStars;
  data->nbStars_p.change_listener(&data->nbStars_p);
  data->maxStars = IVAL(data->nbStars_limit_p);
 
  /*
    if ((IVAL(data->density_p)<10)&&(iRAND(11)<IVAL(data->density_p)))
    addAStar (data,info);
    
    for (i=0;i<(info->screen.height * IVAL(data->density_p) / 10) / 200;++i)
    addAStar (data,info);
  */
    
  if (info->sound.timeSinceLastGoom < 1) {
    int max = (int)((1.0f+info->sound.goomPower)*iRAND(150)) + 100;
    float radius = (1.0f+info->sound.goomPower) * (float)(iRAND(150)+50)/300;
    int mx = iRAND(info->screen.width);
    int my = iRAND(info->screen.height);
    float vage = data->max_age * (1.0f - info->sound.goomPower);

    radius *= info->screen.height / 200.0f; /* FX developped on 320x200 */
    max *= info->screen.height / 200.0f; /* FX developped on 320x200 */

    if (info->sound.timeSinceLastBigGoom < 1) {
      radius *= 1.5;
      max *= 2;
    }
    for (i=0;i<max;++i)
      addABomb (data,mx,my,radius,vage);
  }
  
  for (i=0;i<data->nbStars;++i) {
    updateStar(&data->stars[i]);
    if (data->stars[i].age>=NCOL)
      continue;
    col = colval[(int)data->stars[i].age];
    
    draw_line(dest,(int)data->stars[i].x,(int)data->stars[i].y,
	      (int)(data->stars[i].x-data->stars[i].vx*6),
	      (int)(data->stars[i].y-data->stars[i].vy*6),
	      col,
	      info->screen.width, info->screen.height);
    draw_line(dest,(int)data->stars[i].x,(int)data->stars[i].y,
	      (int)(data->stars[i].x-data->stars[i].vx*2),
	      (int)(data->stars[i].y-data->stars[i].vy*2),
	      col,
	      info->screen.width, info->screen.height);
  }
  for (i=0;i<data->nbStars;) {
    if ((data->stars[i].x > info->screen.width + 64)
	 ||(data->stars[i].y - 16*data->stars[i].vy > info->screen.height)
	 ||(data->stars[i].x < -64)
	 ||(data->stars[i].age>=NCOL)) {
      data->stars[i] = data->stars[data->nbStars-1];
      data->nbStars--;
    }
    else ++i;
  }
}

VisualFX flying_star_create() {
  VisualFX vfx = {
    init: fs_init,
    free: fs_free,
    apply: fs_apply,
    fx_data: 0
  };
  return vfx;
}
