#ifndef _PLUGIN_INFO_H
#define _PLUGIN_INFO_H

typedef struct _PLUGIN_INFO PluginInfo;
typedef struct _SOUND_INFO SoundInfo;

#include "config_param.h"
#include "visual_fx.h"

/**
 * Gives informations about the sound.
 */
struct _SOUND_INFO {
  
  /* nota : a Goom is just a sound event... */

  int timeSinceLastGoom;   /* >= 0 */
  float goomPower;         /* power of the last Goom [0..1] */

  int timeSinceLastBigGoom;   /* >= 0 */

  float volume;     /* [0..1] */

  /* other "internal" datas for the sound_tester */
  float goom_limit; /* auto-updated limit of goom_detection */
  float bigGoomLimit;
  float accelvar;   /* acceleration of the sound - [0..1] */
  float speedvar;   /* speed of the sound - [0..100] */
  int allTimesMax;
  int totalgoom;    /* number of goom since last reset
		     * (a reset every 64 cycles) */

  float prov_max;   /* accel max since last reset */

  int cycle;

  /* private */
  PluginParam volume_p;
  PluginParam speed_p;
  PluginParam accel_p;
  PluginParam goom_limit_p;
  PluginParam last_goom_p;
  PluginParam last_biggoom_p;
  PluginParam biggoom_speed_limit_p;
  PluginParam biggoom_factor_p;

  PluginParameters params; /* contains volume_p,speed_p,accel_p */
};


/**
 * Allows FXs to know the current state of the plugin.
 */
struct _PLUGIN_INFO {

  /* public datas */

  int nbParams;
  PluginParameters *params;

  /* private datas */

  struct _SIZE_TYPE {
    int width;
    int height;
    int size;   /* == screen.height * screen.width. */
  } screen;

  SoundInfo sound;

  int nbVisuals;
  VisualFX **visuals;
};

void plugin_info_init(PluginInfo *p, int nbVisual); /* in sound_tester.c */

/* i = [0..p->nbVisual-1] */
void plugin_info_add_visual(PluginInfo *p, int i, VisualFX *visual);

#endif
