#include "sound_tester.h"

/* some constants */
#define BIG_GOOM_DURATION 100
#define BIG_GOOM_SPEED_LIMIT 0.1f

#define ACCEL_MULT 0.95f
#define SPEED_MULT 0.99f


void evaluate_sound(gint16 data[2][512], SoundInfo *info) {

  int i;
  float difaccel;

  /* find the max */
  int incvar = 0;
  for (i = 0; i < 512; i+=2) {
    if (incvar < data[0][i])
      incvar = data[0][i];
  }

  if (incvar > info->allTimesMax)
    info->allTimesMax = incvar;

  /* volume sonore */
  info->volume = (float)incvar / (float)info->allTimesMax;

  difaccel = info->accelvar;
  info->accelvar = info->volume; /* accel entre 0 et 1 */

  /* transformations sur la vitesse du son */
  if (info->speedvar > 1.0f)
    info->speedvar = 1.0f;
  
  if (info->speedvar < 0.1f)
    info->accelvar *= (1.0f - (float)info->speedvar);
  else if (info->speedvar < 0.3f)
    info->accelvar *= (0.9f - (float)(info->speedvar-0.1f)/2.0f);
  else
    info->accelvar *= (0.8f - (float)(info->speedvar-0.3f)/4.0f);

  /* adoucissement de l'acceleration */
  info->accelvar *= ACCEL_MULT;
  if (info->accelvar < 0)
    info->accelvar = 0;

  difaccel = info->accelvar - difaccel;
  if (difaccel < 0)
    difaccel = - difaccel;

  /* mise a jour de la vitesse */
  info->speedvar = (info->speedvar + difaccel * 0.5f) / 2;
  info->speedvar *= SPEED_MULT;
  if (info->speedvar < 0)
    info->speedvar = 0;
  if (info->speedvar > 1)
    info->speedvar = 1;

  /* temps du goom */
  info->timeSinceLastGoom++;
  info->timeSinceLastBigGoom++;
  info->cycle++;

  /* detection des nouveaux gooms */
  if ((info->speedvar > (float)IVAL(info->biggoom_speed_limit_p)/100.0f)
//      && (info->goom_limit > 0.07)
      && (info->accelvar > info->bigGoomLimit)
      && (info->timeSinceLastBigGoom > BIG_GOOM_DURATION)) {
    info->timeSinceLastBigGoom = 0;
  }

  if (info->accelvar > info->goom_limit) {
    /* TODO: tester && (info->timeSinceLastGoom > 20)) { */
    info->totalgoom ++;
    info->timeSinceLastGoom = 0;
    info->goomPower = info->accelvar - info->goom_limit;    
  }

  if (info->accelvar > info->prov_max)
    info->prov_max = info->accelvar;

  if (info->goom_limit>1)
    info->goom_limit=1;

  /* toute les 2 secondes : vrifier si le taux de goom est correct
   * et le modifier sinon.. */
  if (info->cycle % 64 == 0) {
    if (info->speedvar<0.01f)
      info->goom_limit *= 0.91;
    if (info->totalgoom > 4) {
      info->goom_limit+=0.02;
    }
    if (info->totalgoom > 7) {
      info->goom_limit*=1.03f;
      info->goom_limit+=0.03;
    }
    if (info->totalgoom > 16) {
      info->goom_limit*=1.05f;
      info->goom_limit+=0.04;
    }
    if (info->totalgoom == 0) {
      info->goom_limit = info->prov_max - 0.02;
    }
    if ((info->totalgoom == 1) && (info->goom_limit > 0.02))
      info->goom_limit-=0.01;
    info->totalgoom = 0;
    info->bigGoomLimit = info->goom_limit * (1.0f + (float)IVAL(info->biggoom_factor_p)/500.0f);
    info->prov_max = 0;
  }

  /* mise a jour des parametres pour la GUI */
  FVAL(info->volume_p) = info->volume;
  info->volume_p.change_listener(&info->volume_p);
  FVAL(info->speed_p) = info->speedvar * 4;
  info->speed_p.change_listener(&info->speed_p);
  FVAL(info->accel_p) = info->accelvar;
  info->accel_p.change_listener(&info->accel_p);
  
  FVAL(info->goom_limit_p) = info->goom_limit;
  info->goom_limit_p.change_listener(&info->goom_limit_p);
  FVAL(info->last_goom_p) = 1.0-((float)info->timeSinceLastGoom/20.0f);
  info->last_goom_p.change_listener(&info->last_goom_p);
  FVAL(info->last_biggoom_p) = 1.0-((float)info->timeSinceLastBigGoom/40.0f);
  info->last_biggoom_p.change_listener(&info->last_biggoom_p);

  /* bigGoomLimit ==goomLimit*9/8+7 ? */
}


void plugin_info_init(PluginInfo *pp, int nbVisuals) {
  PluginInfo p;

  p.sound.speedvar =
    p.sound.accelvar =
    p.sound.totalgoom = 0;
  p.sound.goom_limit = 1;
  p.sound.allTimesMax = 1;

  p.sound.volume_p = secure_f_param("Sound Volume");
  p.sound.accel_p = secure_f_param("Sound Acceleration");
  p.sound.speed_p = secure_f_param("Sound Speed");
  p.sound.goom_limit_p = secure_f_param("Goom Limit");
  p.sound.last_goom_p = secure_f_param("Goom Detection");
  p.sound.last_biggoom_p = secure_f_param("Big Goom Detection");

  p.sound.biggoom_speed_limit_p = secure_i_param("Big Goom Speed Limit");
  IVAL(p.sound.biggoom_speed_limit_p) = 10;
  IMIN(p.sound.biggoom_speed_limit_p) = 0;
  IMAX(p.sound.biggoom_speed_limit_p) = 100;
  ISTEP(p.sound.biggoom_speed_limit_p) = 1;

  p.sound.biggoom_factor_p = secure_i_param("Big Goom Factor");
  IVAL(p.sound.biggoom_factor_p) = 10;
  IMIN(p.sound.biggoom_factor_p) = 0;
  IMAX(p.sound.biggoom_factor_p) = 100;
  ISTEP(p.sound.biggoom_factor_p) = 1;

  p.sound.params = plugin_parameters ("Sound", 10);

  p.nbParams = 0;
  p.nbVisuals = nbVisuals;
  p.visuals = (VisualFX**)malloc(sizeof(VisualFX*)*nbVisuals);

  *pp = p;
  pp->sound.params.params[0] = &pp->sound.biggoom_speed_limit_p;
  pp->sound.params.params[1] = &pp->sound.biggoom_factor_p;
  pp->sound.params.params[2] = 0;
  pp->sound.params.params[3] = &pp->sound.volume_p;
  pp->sound.params.params[4] = &pp->sound.accel_p;
  pp->sound.params.params[5] = &pp->sound.speed_p;
  pp->sound.params.params[6] = 0;
  pp->sound.params.params[7] = &pp->sound.goom_limit_p;
  pp->sound.params.params[8] = &pp->sound.last_goom_p; 
  pp->sound.params.params[9] = &pp->sound.last_biggoom_p;

  return p;
}

void plugin_info_add_visual(PluginInfo *p, int i, VisualFX *visual) {
  p->visuals[i] = visual;
  if (i == p->nbVisuals-1) {
    ++i;
    p->nbParams = 1;
    while (i--) {
      if (p->visuals[i]->params)
	p->nbParams++;
    }
    p->params = (PluginParameters *)malloc(sizeof(PluginParameters)*p->nbParams);
    i = p->nbVisuals;
    p->nbParams = 1;
    p->params[0] = p->sound.params;
    while (i--) {
      if (p->visuals[i]->params)
	p->params[p->nbParams++] = *(p->visuals[i]->params);
    }
  }  
}
